//--------------------------------------------------------------------
#include <donut.h>
#include <gl\gl.h>

//--------------------------------------------------------------------
Donut::Donut() : 
	steps_(36),
	center_(0.0, 0.0, 0.0),
	major_axis_(1.0, 0.0, 0.0),
	minor_axis_(0.0, 1.0, 0.0),
	thickness_(0.1),
	color_(Pt3(0.5, 0.5, 0.5))
{
}

//--------------------------------------------------------------------
Donut::~Donut()
{
}

//--------------------------------------------------------------------
void Donut::SetNumSteps(Int s)
{
	steps_ = s;
}

//--------------------------------------------------------------------
void Donut::SetCenter(const Pt3 &c)
{
	center_ = c;
}

//--------------------------------------------------------------------
void Donut::SetMajorAxis(const Pt3 &a)
{
	major_axis_ = a;
}

//--------------------------------------------------------------------
void Donut::SetMinorAxis(const Pt3 &a)
{
	minor_axis_ = a;
}

//--------------------------------------------------------------------
void Donut::SetThickness(Float t)
{
	thickness_ = t;
}

//--------------------------------------------------------------------
void Donut::SetColor(const Pt3 &c)
{
	color_ = c;
}
//--------------------------------------------------------------------
BOOL LoadBMP(TCHAR* szFileName);
//--------------------------------------------------------------------
void Donut::CalcSpine()
{
	BOOL result = LoadBMP("Zork.bmp");
	Int n = GetNumSteps();
	Pt3 c = GetCenter();
	Pt3 u = GetMajorAxis();
	Pt3 v = GetMinorAxis();

	vector<Pt3> &spine = GetSpine();
	vector<Pt3> &tan = GetSpineTan();
	vector<Pt3> &right = GetRights();
	vector<Pt3> &up = GetUps();

	Pt3 up_vect = CrossProduct(u, v).Normalize();

	spine.resize(n+1);
	tan.resize(n+1);
	right.resize(n+1);
	up.resize(n+1);

	for(Int i=0; i<=n; ++i) {
		Float t = (Float)i / (Float)n;
		Float a = t * M_2PI;

		Float ca = cos(a);
		Float sa = sin(a);

		Pt3 x = u * ca + v * sa;
		Pt3 y = u * sa - v * ca;
		Pt3 p = x + y;
		spine[i] = p;
	
		Pt3 xp = - u * sa + v * ca;
		Pt3 yp = u * ca + v * sa;
		Pt3 q = xp + yp;
		tan[i] = q.Normalize();

		Pt3 r = CrossProduct(tan[i], up_vect);

		right[i] = r;
		up[i] = up_vect;
	}
}

//--------------------------------------------------------------------
void Donut::DrawSpine()
{
	vector<Pt3> &spine = GetSpine();
	vector<Pt3> &tan = GetSpineTan();
	vector<Pt3> &right = GetRights();
	vector<Pt3> &up = GetUps();

	int n = spine.size();

	glColor3d(1.0, 0.0, 0.0);
	glBegin(GL_LINE_LOOP);
		for(int i=0; i<n; ++i) {
			glVertex3dv(spine[i].Coord());
		}
	glEnd();

	glColor3d(0.0, 0.5, 1.0);
	glBegin(GL_LINES);
		for(i=0; i<n; ++i) {
			
			Pt3 tmp = spine[i] + tan[i];
			glVertex3dv(spine[i].Coord());
			glVertex3dv(tmp.Coord());

			tmp = spine[i] + right[i];
			glVertex3dv(spine[i].Coord());
			glVertex3dv(tmp.Coord());

			tmp = spine[i] + up[i];
			glVertex3dv(spine[i].Coord());
			glVertex3dv(tmp.Coord());

		}
	glEnd();
}

//--------------------------------------------------------------------
void Donut::CalcRings()
{
	Int n = GetNumSteps();
	Pt3 c = GetCenter();
	Pt3 u = GetMajorAxis();
	Pt3 v = GetMinorAxis();

	Float R = GetThickness();

	vector<Pt3> &spine = GetSpine();
	vector<Pt3> &tan = GetSpineTan();
	vector<Pt3> &right = GetRights();
	vector<Pt3> &up = GetUps();

	vector<vector<Pt3> > &rings = GetRings();
	rings.resize(n+1);

	vector<vector<Pt3> > &normals = GetNormals();
	normals.resize(n+1);

	vector<vector<Pt2> > &texCoords = GetTexCoords();
	texCoords.resize(n+1);

	for(Int k=0; k<=n; ++k) {

		rings[k].resize(n+1);
		normals[k].resize(n+1);
		texCoords[k].resize(n+1);

		for(Int i=0; i<=n; ++i) {
			Float t = (Float)i / (Float)n;
			Float a = t * M_2PI;

			Float ca = cos(a);
			Float sa = sin(a);

			Pt3 x = right[k] * ca + up[k] * sa;
			Pt3 y = right[k] * sa - up[k] * ca;

			Pt3 p = x + y;

			rings[k][i] = spine[k] + (R * p);
			normals[k][i] = p;

			Float tx = Float(k)/Float(n);
			Float ty = Float(i)/Float(n);
			texCoords[k][i] = Pt2(tx,ty);
		}
	}
}

//--------------------------------------------------------------------
void Donut::DrawRings()
{
	vector<vector<Pt3> > &rings = GetRings();

	glColor3d(0.0, 1.0, 0.0);
	for(Int i=0; i<rings.size(); ++i) {
		glBegin(GL_LINE_STRIP);
			for(Int j=0; j<rings[i].size(); ++j) {
				glVertex3dv(rings[i][j].Coord());
			}
		glEnd();
	}

	glColor3d(1.0, 0.0, 0.0);
	for(i=0; i<rings.size(); ++i) {
		glBegin(GL_LINE_STRIP);
			for(Int j=0; j<rings[i].size(); ++j) {
				glVertex3dv(rings[j][i].Coord());
			}
		glEnd();
	}

	vector<vector<Pt3> > &normals = GetNormals();

	glColor3d(1.0, 1.0, 1.0);
	for(i=0; i<normals.size(); ++i) {
		glBegin(GL_LINES);
			for(Int j=0; j<normals[i].size(); ++j) {
				glVertex3dv(rings[i][j].Coord());
				Pt3 tmp = rings[i][j] + normals[i][j];
				glVertex3dv(tmp.Coord());
			}
		glEnd();
	}
}

extern double g_bulge;
//--------------------------------------------------------------------
void Donut::DrawSurface()
{
	//glColor3d(0.0, 1.0, 0.0);

	Material();

	
glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_REPEAT);

glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_REPEAT);

glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, GL_LINEAR);

glTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, GL_LINEAR);

glEnable(GL_TEXTURE_2D);


	vector<vector<Pt3> > &rings = GetRings();
	vector<vector<Pt3> > &normals = GetNormals();
	vector<vector<Pt2> > &texCoords = GetTexCoords();

	
	Int r = rings.size();
	for(Int i=0; i<r-1; ++i) {
		glBegin(GL_QUAD_STRIP);
			Int ri = rings[i].size();
			for(Int j=0; j<ri; ++j) {

				glTexCoord2dv(texCoords[i][j].Coord());
				glNormal3dv(normals[i][j].Coord());
								
				if (j==4 && i==4)
				{
					double x = rings[i][j][0]*g_bulge;
					double y = rings[i][j][1]*g_bulge;
					double z = rings[i][j][2]*g_bulge;

					glVertex3d(x,y,z);
				}
				else
				    glVertex3dv(rings[i][j].Coord());

				glTexCoord2dv(texCoords[i+1][j].Coord());
				glNormal3dv(normals[i+1][j].Coord());
				glVertex3dv(rings[i+1][j].Coord());
			}
		glEnd();
	}
	

	glDisable(GL_LIGHTING);
	glDisable(GL_CULL_FACE);
}

//--------------------------------------------------------------------
void Donut::Draw()
{
	//DrawSpine();
	//DrawRings();
	DrawSurface();
}

//--------------------------------------------------------------------
void Donut::Material()
{
	glEnable(GL_LIGHTING);

	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);

	const Pt3 &c = GetColor();

    /*static*/ GLfloat mat_front_ambient_normal[4] = {c.x(), c.y(), c.z(), 0.5f};
    //static GLfloat mat_front_diffuse_normal[4] = {0.5, 0.9, 0.5, 0.5};
    /*static*/ GLfloat mat_front_diffuse_normal[4] = {c.x(), c.y(), c.z(), 0.5f};

    //static GLfloat mat_back_ambient_normal[4] = {0.9f, 0.1f, 0.1f, 0.5f};
    //static GLfloat mat_back_diffuse_normal[4] = {0.9, 0.5, 0.5, 0.5};
    //static GLfloat mat_back_diffuse_normal[4] = {0.3f, 0.1f, 0.1f, 0.5f};

    //static GLfloat mat_glow[4] = {0.0f, 0.0f, 1.0f, 0.5f};
    static GLfloat mat_null[4] = {0.0f, 0.0f, 0.0f, 0.5f};

    //static GLfloat mat_selected[4] = {1.0f, 0.0f, 0.0f, 0.5f};
    //static GLfloat mat_touched[4] = {0.0f, 0.1f, 0.1f, 0.5f};
    static GLfloat mat_specular[4] = {0.1f, 0.1f, 0.1f, 0.5f};

    static GLfloat mat_shininess[1] = {128.0};

    //glMaterialfv(GL_FRONT, GL_AMBIENT, mat_front_ambient_normal);
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_null);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_front_diffuse_normal);

    //glMaterialfv(GL_BACK, GL_AMBIENT, mat_back_ambient_normal);
    //glMaterialfv(GL_BACK, GL_DIFFUSE, mat_back_diffuse_normal);
    glMaterialfv(GL_FRONT,GL_EMISSION, mat_null);

    //glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
    //glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);

    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

	//glShadeModel(GL_SMOOTH);
	//glShadeModel(GL_FLAT);

	//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	//glPolygonMode(GL_FRONT, GL_FILL);
	//glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
}

//--------------------------------------------------------------------